import typing
import json

from testreportng.result import NGCaseResult


class NGCollector(object):
    def __init__(self):
        self.result: typing.Dict[str, NGCaseResult] = dict()

    def set(self, result: NGCaseResult):
        self.result[result.name] = result

    def get(self, name: str) -> typing.Optional[NGCaseResult]:
        if name not in self.result:
            return None
        return self.result[name]

    def to_dict(self) -> typing.Dict[str, NGCaseResult]:
        return self.result

    def to_json(self) -> str:
        r = dict()
        for name, result in self.result.items():
            r[name] = result.to_json_dict()
        return json.dumps(r)

    def __str__(self):
        return f"<{__class__.__name__} result={self.result}>"

    __repr__ = __str__



import traceback


class NGCaseResult(object):
    STATUS_INIT: str = "init"
    STATUS_PASS: str = "pass"
    STATUS_FAIL: str = "fail"
    STATUS_ERROR: str = "error"
    STATUS_SKIP: str = "skip"

    def __init__(self, name: str, status: str = None):
        if not status:
            status = self.STATUS_INIT
        self.name: str = name
        self.status: str = status

        # data
        self._outcome = None

        # error
        self.error = None
        self.traceback = None

    def to_json_dict(self) -> dict:
        return {
            "name": self.name,
            "status": self.status,
            "error": repr(self.error) if self.error else "",
            "traceback": traceback.format_tb(self.traceback) if self.traceback else "",
        }

    @property
    def outcome(self):
        return self._outcome

    @outcome.setter
    def outcome(self, value):
        error = value.errors[1][1]

        # no error happened
        if not error:
            self.status = self.STATUS_PASS
            return

        # error or fail
        if error[0] is AssertionError:
            self.status = self.STATUS_FAIL
        else:
            self.status = self.STATUS_ERROR

        self.error = error[1]
        self.traceback = error[2]

    def __str__(self):
        return f"<{__class__.__name__} name={self.name} status={self.status}>"

    __repr__ = __str__



import unittest

from testreportng.collector import NGCaseResult, NGCollector


class NGCase(unittest.TestCase):
    # default cls
    ng_result_cls = NGCaseResult
    ng_collector_cls = NGCollector

    # result collector
    ng_result: NGCollector

    @classmethod
    def setUpClass(cls) -> None:
        cls.ng_result: NGCollector = cls.ng_collector_cls()

    def setUp(self) -> None:
        cur = self.ng_result_cls(self._testMethodName)
        self.ng_result.set(cur)

    def tearDown(self) -> None:
        cur = self.ng_result.get(self._testMethodName)
        cur.outcome = getattr(self, "_outcome")

        self.ng_result.set(cur)
